// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "component/display/material/Material.hpp"
#include "display/shader/ShaderData.hpp"

namespace Dawn {
  struct SimpleTexturedMaterialShaderData : public ShaderData {
    protected:
      void writeData() override {
        this->beginStruct();
        // this->writeMat4(this->projection);
        // this->writeMat4(this->view);
        // this->writeMat4(this->model);
        // this->writeColor(this->color);
        // this->writeBoolean(this->hasTexture);
        // this->writeTexture(this->texture);
        this->endStruct();
      }

    public:
      struct Color color;
      glm::mat4 model;
      glm::mat4 projection;
      glm::mat4 view;
      bool hasTexture;
      std::shared_ptr<Texture> texture;
  };

  class SimpleTexturedMaterial : public Material {
    private:
      struct SimpleTexturedMaterialShaderData data;
      std::shared_ptr<ShaderProgram> shader;

    protected:
      void initShaderPrograms() override;

    public:
      void load(std::shared_ptr<SceneLoadContext> ctx) override;
      
      /**
       * Returns the color of this material.
       */
      struct Color getColor();

      /**
       * Returns the texture of this material.
       * 
       * @return The texture of this material.
       */
      std::shared_ptr<Texture> getTexture();

      /**
       * Sets the texture of this material.
       * 
       * @param texture The texture to set.
       */
      void setTexture(const std::shared_ptr<Texture> texture);

      /**
       * Sets the color of this material.
       * 
       * @param color The color to set.
       */
      void setColor(const struct Color color);

      std::vector<std::shared_ptr<RenderPass>> getPasses(
        struct RenderPassContext &ctx
      ) override;
  };
}