// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/Texture.hpp"
#include "display/shader/ShaderProgram.hpp"

namespace Dawn {
  class IShaderData {
    private:
      std::shared_ptr<ShaderProgram> shader;

    protected:
      virtual void writeData() = 0;

      virtual void beginStruct() = 0;
      virtual void writeMat4(const glm::mat4 &mat) = 0;
      virtual void writeVec3(const glm::vec3 &vec) = 0;
      virtual void writeVec4(const glm::vec4 &vec) = 0;
      virtual void writeColor(const struct Color &color) = 0;
      virtual void writeBoolean(const bool &value) = 0;
      virtual void writeTexture(std::shared_ptr<Texture> &texture) = 0;
      virtual void endStruct() = 0;

    public:
      /**
       * Writes the data to the shader.
       * 
       * @param shader The shader to write to.
       */
      void write(std::shared_ptr<ShaderProgram> shader);

      /**
       * Returns the shader that was written to.
       * 
       * @return The shader that was written to.
       */
      std::shared_ptr<ShaderProgram> getShader();
  };
}